"""
Copyright (c) 2023 Akhil Sharma. All rights reserved.

pystocktopus: help you maintain your stock dashboard and also predict future for a stock based upon past data
"""


from __future__ import annotations
